@@ -371,7 +371,7 @@ struct Config {
 
   // check = >=0.0
   // desc = used for the categorical features
-  // desc = L2 regularization in categorcial split
+  // desc = L2 regularization in categorical split
   double cat_l2 = 10.0;
 
   // check = >=0.0
@@ -551,7 +551,7 @@ struct Config {
   // desc = set this to ``false`` to disable the special handle of missing value
   bool use_missing = true;
 
-  // desc = set this to ``true`` to treat all zero as missing values (including the unshown values in libsvm/sparse matrices)
+  // desc = set this to ``true`` to treat all zero as missing values (including the unshown values in LibSVM / sparse matrices)
   // desc = set this to ``false`` to use ``na`` for representing missing values
   bool zero_as_missing = false;
 
@@ -964,6 +964,59 @@ struct ParameterAlias {
   }
 };
 
+inline std::string ParseObjectiveAlias(const std::string& type) {
+  if (type == std::string("regression") || type == std::string("regression_l2")
+    || type == std::string("mean_squared_error") || type == std::string("mse") || type == std::string("l2")
+    || type == std::string("l2_root") || type == std::string("root_mean_squared_error") || type == std::string("rmse")) {
+    return "regression";
+  } else if (type == std::string("regression_l1") || type == std::string("mean_absolute_error")
+    || type == std::string("l1") || type == std::string("mae")) {
+    return "regression_l1";
+  } else if (type == std::string("multiclass") || type == std::string("softmax")) {
+    return "multiclass";
+  } else if (type == std::string("multiclassova") || type == std::string("multiclass_ova") || type == std::string("ova") || type == std::string("ovr")) {
+    return "multiclassova";
+  } else if (type == std::string("xentropy") || type == std::string("cross_entropy")) {
+    return "cross_entropy";
+  } else if (type == std::string("xentlambda") || type == std::string("cross_entropy_lambda")) {
+    return "cross_entropy_lambda";
+  } else if (type == std::string("mean_absolute_percentage_error") || type == std::string("mape")) {
+    return "mape";
+  } else if (type == std::string("none") || type == std::string("null") || type == std::string("custom") || type == std::string("na")) {
+    return "custom";
+  }
+  return type;
+}
+
+inline std::string ParseMetricAlias(const std::string& type) {
+  if (type == std::string("regression") || type == std::string("regression_l2") || type == std::string("l2") || type == std::string("mean_squared_error") || type == std::string("mse")) {
+    return "l2";
+  } else if (type == std::string("l2_root") || type == std::string("root_mean_squared_error") || type == std::string("rmse")) {
+    return "rmse";
+  } else if (type == std::string("regression_l1") || type == std::string("l1") || type == std::string("mean_absolute_error") || type == std::string("mae")) {
+    return "l1";
+  } else if (type == std::string("binary_logloss") || type == std::string("binary")) {
+    return "binary_logloss";
+  } else if (type == std::string("ndcg") || type == std::string("lambdarank")) {
+    return "ndcg";
+  } else if (type == std::string("map") || type == std::string("mean_average_precision")) {
+    return "map";
+  } else if (type == std::string("multi_logloss") || type == std::string("multiclass") || type == std::string("softmax") || type == std::string("multiclassova") || type == std::string("multiclass_ova") || type == std::string("ova") || type == std::string("ovr")) {
+    return "multi_logloss";
+  } else if (type == std::string("xentropy") || type == std::string("cross_entropy")) {
+    return "cross_entropy";
+  } else if (type == std::string("xentlambda") || type == std::string("cross_entropy_lambda")) {
+    return "cross_entropy_lambda";
+  } else if (type == std::string("kldiv") || type == std::string("kullback_leibler")) {
+    return "kullback_leibler";
+  } else if (type == std::string("mean_absolute_percentage_error") || type == std::string("mape")) {
+    return "mape";
+  } else if (type == std::string("none") || type == std::string("null") || type == std::string("custom") || type == std::string("na")) {
+    return "custom";
+  }
+  return type;
+}
+
 }   // namespace LightGBM
 
-#endif   // LightGBM_CONFIG_H_
+#endif   // LightGBM_CONFIG_H_
\ No newline at end of file
