@@ -411,7 +411,8 @@ struct __TToStringHelperFast<T, true, false> {
                   #ifdef _MSC_VER
                   buf_len
                   #endif
-                  ) const {
+                  )
+  const {
     #ifdef _MSC_VER
     sprintf_s(buffer, buf_len, "%g", value);
     #else
@@ -595,7 +596,7 @@ inline static int64_t Pow2RoundUp(int64_t x) {
 }
 
 /*!
- * \brief Do inplace softmax transformaton on p_rec
+ * \brief Do inplace softmax transformation on p_rec
  * \param p_rec The input/output vector of the values.
  */
 inline static void Softmax(std::vector<double>* p_rec) {
@@ -632,8 +633,8 @@ inline static void Softmax(const double* input, double* output, int len) {
 template<typename T>
 std::vector<const T*> ConstPtrInVectorWrapper(const std::vector<std::unique_ptr<T>>& input) {
   std::vector<const T*> ret;
-  for (size_t i = 0; i < input.size(); ++i) {
-    ret.push_back(input.at(i).get());
+  for (auto t = input.begin(); t !=input.end(); ++t) {
+    ret.push_back(t->get());
   }
   return ret;
 }
@@ -641,8 +642,10 @@ std::vector<const T*> ConstPtrInVectorWrapper(const std::vector<std::unique_ptr<
 template<typename T1, typename T2>
 inline static void SortForPair(std::vector<T1>* keys, std::vector<T2>* values, size_t start, bool is_reverse = false) {
   std::vector<std::pair<T1, T2>> arr;
+  auto& ref_key = *keys;
+  auto& ref_value = *values;
   for (size_t i = start; i < keys->size(); ++i) {
-    arr.emplace_back(keys->at(i), values->at(i));
+    arr.emplace_back(ref_key[i], ref_value[i]);
   }
   if (!is_reverse) {
     std::stable_sort(arr.begin(), arr.end(), [](const std::pair<T1, T2>& a, const std::pair<T1, T2>& b) {
@@ -654,16 +657,17 @@ inline static void SortForPair(std::vector<T1>* keys, std::vector<T2>* values, s
     });
   }
   for (size_t i = start; i < arr.size(); ++i) {
-    keys->at(i) = arr[i].first;
-    values->at(i) = arr[i].second;
+    ref_key[i] = arr[i].first;
+    ref_value[i] = arr[i].second;
   }
 }
 
 template <typename T>
 inline static std::vector<T*> Vector2Ptr(std::vector<std::vector<T>>* data) {
   std::vector<T*> ptr(data->size());
+  auto& ref_data = *data;
   for (size_t i = 0; i < data->size(); ++i) {
-    ptr[i] = data->at(i).data();
+    ptr[i] = ref_data[i].data();
   }
   return ptr;
 }
@@ -841,12 +845,13 @@ inline static std::vector<uint32_t> EmptyBitset(int n) {
 
 template<typename T>
 inline static void InsertBitset(std::vector<uint32_t>* vec, const T val) {
-    int i1 = val / 32;
-    int i2 = val % 32;
-    if (static_cast<int>(vec->size()) < i1 + 1) {
-      vec->resize(i1 + 1, 0);
-    }
-    vec->at(i1) |= (1 << i2);
+  auto& ref_v = *vec;
+  int i1 = val / 32;
+  int i2 = val % 32;
+  if (static_cast<int>(vec->size()) < i1 + 1) {
+    vec->resize(i1 + 1, 0);
+  }
+  ref_v[i1] |= (1 << i2);
 }
 
 template<typename T>
@@ -923,6 +928,24 @@ inline bool CheckASCII(const std::string& s) {
   return true;
 }
 
+inline bool CheckAllowedJSON(const std::string& s) {
+  unsigned char char_code;
+  for (auto c : s) {
+    char_code = static_cast<unsigned char>(c);
+    if (char_code == 34      // "
+        || char_code == 44   // ,
+        || char_code == 58   // :
+        || char_code == 91   // [
+        || char_code == 93   // ]
+        || char_code == 123  // {
+        || char_code == 125  // }
+        ) {
+      return false;
+    }
+  }
+  return true;
+}
+
 }  // namespace Common
 
 }  // namespace LightGBM
