@@ -61,8 +61,9 @@ int GetConfilctCount(const std::vector<bool>& mark, const int* indices, int num_
   return ret;
 }
 void MarkUsed(std::vector<bool>* mark, const int* indices, int num_indices) {
+  auto& ref_mark = *mark;
   for (int i = 0; i < num_indices; ++i) {
-    mark->at(indices[i]) = true;
+    ref_mark[indices[i]] = true;
   }
 }
 
@@ -70,6 +71,7 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
                                          const std::vector<int>& find_order,
                                          int** sample_indices,
                                          const int* num_per_col,
+                                         int num_sample_col,
                                          size_t total_sample_cnt,
                                          data_size_t max_error_cnt,
                                          data_size_t filter_cnt,
@@ -85,7 +87,8 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
   std::vector<int> group_num_bin;
 
   for (auto fidx : find_order) {
-    const size_t cur_non_zero_cnt = num_per_col[fidx];
+    bool is_filtered_feature = fidx >= num_sample_col;
+    const size_t cur_non_zero_cnt = is_filtered_feature ? 0: num_per_col[fidx];
     bool need_new_group = true;
     std::vector<int> available_groups;
     for (int gid = 0; gid < static_cast<int>(features_in_group.size()); ++gid) {
@@ -107,7 +110,7 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
     }
     for (auto gid : search_groups) {
       const int rest_max_cnt = max_error_cnt - group_conflict_cnt[gid];
-      int cnt = GetConfilctCount(conflict_marks[gid], sample_indices[fidx], num_per_col[fidx], rest_max_cnt);
+      const int cnt = is_filtered_feature ? 0 : GetConfilctCount(conflict_marks[gid], sample_indices[fidx], num_per_col[fidx], rest_max_cnt);
       if (cnt >= 0 && cnt <= rest_max_cnt) {
         data_size_t rest_non_zero_data = static_cast<data_size_t>(
           static_cast<double>(cur_non_zero_cnt - cnt) * num_data / total_sample_cnt);
@@ -116,7 +119,9 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
         features_in_group[gid].push_back(fidx);
         group_conflict_cnt[gid] += cnt;
         group_non_zero_cnt[gid] += cur_non_zero_cnt - cnt;
-        MarkUsed(&conflict_marks[gid], sample_indices[fidx], num_per_col[fidx]);
+        if (!is_filtered_feature) {
+          MarkUsed(&conflict_marks[gid], sample_indices[fidx], num_per_col[fidx]);
+        }
         if (is_use_gpu) {
           group_num_bin[gid] += bin_mappers[fidx]->num_bin() + (bin_mappers[fidx]->GetDefaultBin() == 0 ? -1 : 0);
         }
@@ -128,7 +133,9 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
       features_in_group.back().push_back(fidx);
       group_conflict_cnt.push_back(0);
       conflict_marks.emplace_back(total_sample_cnt, false);
-      MarkUsed(&(conflict_marks.back()), sample_indices[fidx], num_per_col[fidx]);
+      if (!is_filtered_feature) {
+        MarkUsed(&(conflict_marks.back()), sample_indices[fidx], num_per_col[fidx]);
+      }
       group_non_zero_cnt.emplace_back(cur_non_zero_cnt);
       if (is_use_gpu) {
         group_num_bin.push_back(1 + bin_mappers[fidx]->num_bin() + (bin_mappers[fidx]->GetDefaultBin() == 0 ? -1 : 0));
@@ -141,6 +148,7 @@ std::vector<std::vector<int>> FindGroups(const std::vector<std::unique_ptr<BinMa
 std::vector<std::vector<int>> FastFeatureBundling(const std::vector<std::unique_ptr<BinMapper>>& bin_mappers,
                                                   int** sample_indices,
                                                   const int* num_per_col,
+                                                  int num_sample_col,
                                                   size_t total_sample_cnt,
                                                   const std::vector<int>& used_features,
                                                   double max_conflict_rate,
@@ -156,7 +164,11 @@ std::vector<std::vector<int>> FastFeatureBundling(const std::vector<std::unique_
   feature_non_zero_cnt.reserve(used_features.size());
   // put dense feature first
   for (auto fidx : used_features) {
-    feature_non_zero_cnt.emplace_back(num_per_col[fidx]);
+    if (fidx < num_sample_col) {
+      feature_non_zero_cnt.emplace_back(num_per_col[fidx]);
+    } else {
+      feature_non_zero_cnt.emplace_back(0);
+    }
   }
   // sort by non zero cnt
   std::vector<int> sorted_idx;
@@ -175,8 +187,8 @@ std::vector<std::vector<int>> FastFeatureBundling(const std::vector<std::unique_
   for (auto sidx : sorted_idx) {
     feature_order_by_cnt.push_back(used_features[sidx]);
   }
-  auto features_in_group = FindGroups(bin_mappers, used_features, sample_indices, num_per_col, total_sample_cnt, max_error_cnt, filter_cnt, num_data, is_use_gpu);
-  auto group2 = FindGroups(bin_mappers, feature_order_by_cnt, sample_indices, num_per_col, total_sample_cnt, max_error_cnt, filter_cnt, num_data, is_use_gpu);
+  auto features_in_group = FindGroups(bin_mappers, used_features, sample_indices, num_per_col, num_sample_col, total_sample_cnt, max_error_cnt, filter_cnt, num_data, is_use_gpu);
+  auto group2 = FindGroups(bin_mappers, feature_order_by_cnt, sample_indices, num_per_col, num_sample_col, total_sample_cnt, max_error_cnt, filter_cnt, num_data, is_use_gpu);
   if (features_in_group.size() > group2.size()) {
     features_in_group = group2;
   }
@@ -215,17 +227,21 @@ std::vector<std::vector<int>> FastFeatureBundling(const std::vector<std::unique_
 
 void Dataset::Construct(
   std::vector<std::unique_ptr<BinMapper>>* bin_mappers,
+  int num_total_features,
   const std::vector<std::vector<double>>& forced_bins,
   int** sample_non_zero_indices,
   const int* num_per_col,
+  int num_sample_col,
   size_t total_sample_cnt,
   const Config& io_config) {
-  num_total_features_ = static_cast<int>(bin_mappers->size());
+  num_total_features_ = num_total_features;
+  CHECK(num_total_features_ == static_cast<int>(bin_mappers->size()));
   sparse_threshold_ = io_config.sparse_threshold;
   // get num_features
   std::vector<int> used_features;
+  auto& ref_bin_mappers = *bin_mappers;
   for (int i = 0; i < static_cast<int>(bin_mappers->size()); ++i) {
-    if (bin_mappers->at(i) != nullptr && !bin_mappers->at(i)->is_trivial()) {
+    if (ref_bin_mappers[i] != nullptr && !ref_bin_mappers[i]->is_trivial()) {
       used_features.emplace_back(i);
     }
   }
@@ -236,7 +252,7 @@ void Dataset::Construct(
 
   if (io_config.enable_bundle && !used_features.empty()) {
     features_in_group = FastFeatureBundling(*bin_mappers,
-                                            sample_non_zero_indices, num_per_col, total_sample_cnt,
+                                            sample_non_zero_indices, num_per_col, num_sample_col, total_sample_cnt,
                                             used_features, io_config.max_conflict_rate,
                                             num_data_, io_config.min_data_in_leaf,
                                             sparse_threshold_, io_config.is_enable_sparse, io_config.device_type == std::string("gpu"));
@@ -263,7 +279,7 @@ void Dataset::Construct(
       real_feature_idx_[cur_fidx] = real_fidx;
       feature2group_[cur_fidx] = i;
       feature2subfeature_[cur_fidx] = j;
-      cur_bin_mappers.emplace_back(bin_mappers->at(real_fidx).release());
+      cur_bin_mappers.emplace_back(ref_bin_mappers[real_fidx].release());
       ++cur_fidx;
     }
     feature_groups_.emplace_back(std::unique_ptr<FeatureGroup>(
@@ -721,7 +737,7 @@ void Dataset::SaveBinaryFile(const char* bin_filename) {
     for (int i = 0; i < num_total_features_; ++i) {
       int num_bounds = static_cast<int>(forced_bin_bounds_[i].size());
       writer->Write(&num_bounds, sizeof(int));
-      
+
       for (size_t j = 0; j < forced_bin_bounds_[i].size(); ++j) {
         writer->Write(&forced_bin_bounds_[i][j], sizeof(double));
       }
@@ -834,6 +850,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
   int num_used_group = static_cast<int>(used_group.size());
   auto ptr_ordered_grad = gradients;
   auto ptr_ordered_hess = hessians;
+  auto& ref_ordered_bins = *ordered_bins;
   if (data_indices != nullptr && num_data < num_data_) {
     if (!is_constant_hessian) {
       #pragma omp parallel for schedule(static)
@@ -860,7 +877,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         const int num_bin = feature_groups_[group]->num_total_bin_;
         std::memset(reinterpret_cast<void*>(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
-        if (ordered_bins->at(group) == nullptr) {
+        if (ref_ordered_bins[group] == nullptr) {
           // if not use ordered bin
           feature_groups_[group]->bin_data_->ConstructHistogram(
             data_indices,
@@ -870,10 +887,10 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
             data_ptr);
         } else {
           // used ordered bin
-          ordered_bins->at(group)->ConstructHistogram(leaf_idx,
-                                                  gradients,
-                                                  hessians,
-                                                  data_ptr);
+          ref_ordered_bins[group]->ConstructHistogram(leaf_idx,
+                                                      gradients,
+                                                      hessians,
+                                                      data_ptr);
         }
         OMP_LOOP_EX_END();
       }
@@ -889,7 +906,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         const int num_bin = feature_groups_[group]->num_total_bin_;
         std::memset(reinterpret_cast<void*>(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
-        if (ordered_bins->at(group) == nullptr) {
+        if (ref_ordered_bins[group] == nullptr) {
           // if not use ordered bin
           feature_groups_[group]->bin_data_->ConstructHistogram(
             data_indices,
@@ -898,9 +915,9 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
             data_ptr);
         } else {
           // used ordered bin
-          ordered_bins->at(group)->ConstructHistogram(leaf_idx,
-                                                  gradients,
-                                                  data_ptr);
+          ref_ordered_bins[group]->ConstructHistogram(leaf_idx,
+                                                      gradients,
+                                                      data_ptr);
         }
         // fixed hessian.
         for (int i = 0; i < num_bin; ++i) {
@@ -922,7 +939,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         const int num_bin = feature_groups_[group]->num_total_bin_;
         std::memset(reinterpret_cast<void*>(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
-        if (ordered_bins->at(group) == nullptr) {
+        if (ref_ordered_bins[group] == nullptr) {
           // if not use ordered bin
           feature_groups_[group]->bin_data_->ConstructHistogram(
             num_data,
@@ -931,10 +948,10 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
             data_ptr);
         } else {
           // used ordered bin
-          ordered_bins->at(group)->ConstructHistogram(leaf_idx,
-                                                  gradients,
-                                                  hessians,
-                                                  data_ptr);
+          ref_ordered_bins[group]->ConstructHistogram(leaf_idx,
+                                                      gradients,
+                                                      hessians,
+                                                      data_ptr);
         }
         OMP_LOOP_EX_END();
       }
@@ -950,7 +967,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         const int num_bin = feature_groups_[group]->num_total_bin_;
         std::memset(reinterpret_cast<void*>(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
-        if (ordered_bins->at(group) == nullptr) {
+        if (ref_ordered_bins[group] == nullptr) {
           // if not use ordered bin
           feature_groups_[group]->bin_data_->ConstructHistogram(
             num_data,
@@ -958,9 +975,9 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
             data_ptr);
         } else {
           // used ordered bin
-          ordered_bins->at(group)->ConstructHistogram(leaf_idx,
-                                                  gradients,
-                                                  data_ptr);
+          ref_ordered_bins[group]->ConstructHistogram(leaf_idx,
+                                                      gradients,
+                                                      data_ptr);
         }
         // fixed hessian.
         for (int i = 0; i < num_bin; ++i) {
