@@ -31,9 +31,9 @@ class SparseBinIterator: public BinIterator {
     max_bin_(static_cast<VAL_T>(max_bin)),
     default_bin_(static_cast<VAL_T>(default_bin)) {
     if (default_bin_ == 0) {
-      bias_ = 1;
+      offset_ = 1;
     } else {
-      bias_ = 0;
+      offset_ = 0;
     }
     Reset(0);
   }
@@ -48,7 +48,7 @@ class SparseBinIterator: public BinIterator {
   inline uint32_t Get(data_size_t idx) override {
     VAL_T ret = InnerRawGet(idx);
     if (ret >= min_bin_ && ret <= max_bin_) {
-      return ret - min_bin_ + bias_;
+      return ret - min_bin_ + offset_;
     } else {
       return default_bin_;
     }
@@ -63,7 +63,7 @@ class SparseBinIterator: public BinIterator {
   VAL_T min_bin_;
   VAL_T max_bin_;
   VAL_T default_bin_;
-  uint8_t bias_;
+  uint8_t offset_;
 };
 
 template <typename VAL_T>
