@@ -1,10 +1,18 @@
+/*!
+ * Copyright (c) 2016 Microsoft Corporation. All rights reserved.
+ * Licensed under the MIT License. See LICENSE file in the project root for license information.
+ */
 #ifndef LIGHTGBM_OBJECTIVE_MULTICLASS_OBJECTIVE_HPP_
 #define LIGHTGBM_OBJECTIVE_MULTICLASS_OBJECTIVE_HPP_
 
+#include <LightGBM/network.h>
 #include <LightGBM/objective_function.h>
 
-#include <cstring>
+#include <string>
+#include <algorithm>
 #include <cmath>
+#include <cstring>
+#include <memory>
 #include <vector>
 
 #include "binary_objective.hpp"
@@ -14,7 +22,7 @@ namespace LightGBM {
 * \brief Objective function for multiclass classification, use softmax as objective functions
 */
 class MulticlassSoftmax: public ObjectiveFunction {
-public:
+ public:
   explicit MulticlassSoftmax(const Config& config) {
     num_class_ = config.num_class;
   }
@@ -35,7 +43,6 @@ public:
   }
 
   ~MulticlassSoftmax() {
-
   }
 
   void Init(const Metadata& metadata, data_size_t num_data) override {
@@ -60,6 +67,12 @@ public:
     if (weights_ == nullptr) {
       sum_weight = num_data_;
     }
+    if (Network::num_machines() > 1) {
+      sum_weight = Network::GlobalSyncUpBySum(sum_weight);
+      for (int i = 0; i < num_class_; ++i) {
+        class_init_probs_[i] = Network::GlobalSyncUpBySum(class_init_probs_[i]);
+      }
+    }
     for (int i = 0; i < num_class_; ++i) {
       class_init_probs_[i] /= sum_weight;
     }
@@ -138,8 +151,8 @@ public:
     return std::log(std::max<double>(kEpsilon, class_init_probs_[class_id]));
   }
 
-  bool ClassNeedTrain(int class_id) const override { 
-    if (std::fabs(class_init_probs_[class_id]) <= kEpsilon 
+  bool ClassNeedTrain(int class_id) const override {
+    if (std::fabs(class_init_probs_[class_id]) <= kEpsilon
         || std::fabs(class_init_probs_[class_id]) >= 1.0 - kEpsilon) {
       return false;
     } else {
@@ -147,7 +160,7 @@ public:
     }
   }
 
-private:
+ private:
   /*! \brief Number of data */
   data_size_t num_data_;
   /*! \brief Number of classes */
@@ -165,7 +178,7 @@ private:
 * \brief Objective function for multiclass classification, use one-vs-all binary objective function
 */
 class MulticlassOVA: public ObjectiveFunction {
-public:
+ public:
   explicit MulticlassOVA(const Config& config) {
     num_class_ = config.num_class;
     for (int i = 0; i < num_class_; ++i) {
@@ -197,7 +210,6 @@ public:
   }
 
   ~MulticlassOVA() {
-
   }
 
   void Init(const Metadata& metadata, data_size_t num_data) override {
@@ -209,8 +221,8 @@ public:
 
   void GetGradients(const double* score, score_t* gradients, score_t* hessians) const override {
     for (int i = 0; i < num_class_; ++i) {
-      int64_t bias = static_cast<int64_t>(num_data_) * i;
-      binary_loss_[i]->GetGradients(score + bias, gradients + bias, hessians + bias);
+      int64_t offset = static_cast<int64_t>(num_data_) * i;
+      binary_loss_[i]->GetGradients(score + offset, gradients + offset, hessians + offset);
     }
   }
 
@@ -248,7 +260,7 @@ public:
     return binary_loss_[class_id]->ClassNeedTrain(0);
   }
 
-private:
+ private:
   /*! \brief Number of data */
   data_size_t num_data_;
   /*! \brief Number of classes */
