@@ -1,23 +1,28 @@
+/*!
+ * Copyright (c) 2016 Microsoft Corporation. All rights reserved.
+ * Licensed under the MIT License. See LICENSE file in the project root for license information.
+ */
 #ifndef LIGHTGBM_TREELEARNER_SERIAL_TREE_LEARNER_H_
 #define LIGHTGBM_TREELEARNER_SERIAL_TREE_LEARNER_H_
-#include <LightGBM/tree_learner.h>
-
-#include <LightGBM/utils/random.h>
-#include <LightGBM/utils/array_args.h>
 
 #include <LightGBM/dataset.h>
 #include <LightGBM/tree.h>
+#include <LightGBM/tree_learner.h>
+#include <LightGBM/utils/array_args.h>
+#include <LightGBM/utils/random.h>
 
-#include "feature_histogram.hpp"
-#include "split_info.hpp"
-#include "data_partition.hpp"
-#include "leaf_splits.hpp"
-
-#include <cstdio>
-#include <vector>
-#include <random>
+#include <string>
 #include <cmath>
+#include <cstdio>
 #include <memory>
+#include <random>
+#include <vector>
+
+#include "data_partition.hpp"
+#include "feature_histogram.hpp"
+#include "leaf_splits.hpp"
+#include "split_info.hpp"
+
 #ifdef USE_GPU
 // Use 4KBytes aligned allocator for ordered gradients and ordered hessians when GPU is enabled.
 // This is necessary to pin the two arrays in memory and make transferring faster.
@@ -27,12 +32,14 @@
 using namespace json11;
 
 namespace LightGBM {
-
+/*! \brief forward declaration */
+class CostEfficientGradientBoosting;
 /*!
 * \brief Used for learning a tree by single machine
 */
 class SerialTreeLearner: public TreeLearner {
-public:
+ public:
+  friend CostEfficientGradientBoosting;
   explicit SerialTreeLearner(const Config* config);
 
   ~SerialTreeLearner();
@@ -44,7 +51,7 @@ public:
   void ResetConfig(const Config* config) override;
 
   Tree* Train(const score_t* gradients, const score_t *hessians, bool is_constant_hessian,
-              Json& forced_split_json) override;
+              const Json& forced_split_json) override;
 
   Tree* FitByExistingTree(const Tree* old_tree, const score_t* gradients, const score_t* hessians) const override;
 
@@ -69,10 +76,11 @@ public:
     }
   }
 
-  void RenewTreeOutput(Tree* tree, const ObjectiveFunction* obj, const double* prediction,
+  void RenewTreeOutput(Tree* tree, const ObjectiveFunction* obj, std::function<double(const label_t*, int)> residual_getter,
                        data_size_t total_num_data, const data_size_t* bag_indices, data_size_t bag_cnt) const override;
 
-protected:
+ protected:
+  virtual std::vector<int8_t> GetUsedFeatures(bool is_tree_level);
   /*!
   * \brief Some initial works before training
   */
@@ -99,17 +107,17 @@ protected:
   virtual void Split(Tree* tree, int best_leaf, int* left_leaf, int* right_leaf);
 
   /* Force splits with forced_split_json dict and then return num splits forced.*/
-  virtual int32_t ForceSplits(Tree* tree, Json& forced_split_json, int* left_leaf,
-                              int* right_leaf, int* cur_depth, 
+  virtual int32_t ForceSplits(Tree* tree, const Json& forced_split_json, int* left_leaf,
+                              int* right_leaf, int* cur_depth,
                               bool *aborted_last_force_split);
 
-
   /*!
   * \brief Get the number of data in a leaf
   * \param leaf_idx The index of leaf
   * \return The number of data in the leaf_idx leaf
   */
   inline virtual data_size_t GetGlobalDataCountInLeaf(int leaf_idx) const;
+
   /*! \brief number of data */
   data_size_t num_data_;
   /*! \brief number of features */
@@ -126,6 +134,8 @@ protected:
   Random random_;
   /*! \brief used for sub feature training, is_feature_used_[i] = false means don't used feature i */
   std::vector<int8_t> is_feature_used_;
+  /*! \brief used feature indices in current tree */
+  std::vector<int> used_feature_indices_;
   /*! \brief pointer to histograms array of parent of current leaves */
   FeatureHistogram* parent_leaf_histogram_array_;
   /*! \brief pointer to histograms array of smaller leaf */
@@ -135,6 +145,8 @@ protected:
 
   /*! \brief store best split points for all leaves */
   std::vector<SplitInfo> best_split_per_leaf_;
+  /*! \brief store best split per feature for all leaves */
+  std::vector<SplitInfo> splits_per_leaf_;
 
   /*! \brief stores best thresholds for all feature for smaller leaf */
   std::unique_ptr<LeafSplits> smaller_leaf_splits_;
@@ -167,6 +179,7 @@ protected:
   int num_threads_;
   std::vector<int> ordered_bin_indices_;
   bool is_constant_hessian_;
+  std::unique_ptr<CostEfficientGradientBoosting> cegb_;
 };
 
 inline data_size_t SerialTreeLearner::GetGlobalDataCountInLeaf(int leaf_idx) const {
