@@ -1,6 +1,5 @@
 # coding: utf-8
-# pylint: disable = C0103
-"""Compatibility"""
+"""Compatibility library."""
 from __future__ import absolute_import
 
 import inspect
@@ -10,7 +9,7 @@ import numpy as np
 
 is_py3 = (sys.version_info[0] == 3)
 
-"""compatibility between python2 and python3"""
+"""Compatibility between Python2 and Python3"""
 if is_py3:
     zip_ = zip
     string_type = str
@@ -19,10 +18,11 @@ if is_py3:
     range_ = range
 
     def argc_(func):
-        """return number of arguments of a function"""
+        """Count the number of arguments of a function."""
         return len(inspect.signature(func).parameters)
 
     def decode_string(bytestring):
+        """Decode C bytestring to ordinary string."""
         return bytestring.decode('utf-8')
 else:
     from itertools import izip as zip_
@@ -32,10 +32,11 @@ else:
     range_ = xrange
 
     def argc_(func):
-        """return number of arguments of a function"""
+        """Count the number of arguments of a function."""
         return len(inspect.getargspec(func).args)
 
     def decode_string(bytestring):
+        """Decode C bytestring to ordinary string."""
         return bytestring
 
 """json"""
@@ -48,6 +49,7 @@ except (ImportError, SyntaxError):
 
 
 def json_default_with_numpy(obj):
+    """Convert numpy classes to JSON serializable objects."""
     if isinstance(obj, (np.integer, np.floating, np.bool_)):
         return obj.item()
     elif isinstance(obj, np.ndarray):
@@ -59,16 +61,23 @@ def json_default_with_numpy(obj):
 """pandas"""
 try:
     from pandas import Series, DataFrame
+    from pandas.api.types import is_sparse as is_dtype_sparse
     PANDAS_INSTALLED = True
 except ImportError:
     PANDAS_INSTALLED = False
 
     class Series(object):
+        """Dummy class for pandas.Series."""
+
         pass
 
     class DataFrame(object):
+        """Dummy class for pandas.DataFrame."""
+
         pass
 
+    is_dtype_sparse = None
+
 """matplotlib"""
 try:
     import matplotlib
@@ -83,6 +92,23 @@ try:
 except ImportError:
     GRAPHVIZ_INSTALLED = False
 
+"""datatable"""
+try:
+    import datatable
+    if hasattr(datatable, "Frame"):
+        DataTable = datatable.Frame
+    else:
+        DataTable = datatable.DataTable
+    DATATABLE_INSTALLED = True
+except ImportError:
+    DATATABLE_INSTALLED = False
+
+    class DataTable(object):
+        """Dummy class for DataTable."""
+
+        pass
+
+
 """sklearn"""
 try:
     from sklearn.base import BaseEstimator
@@ -90,7 +116,8 @@ try:
     from sklearn.preprocessing import LabelEncoder
     from sklearn.utils.class_weight import compute_sample_weight
     from sklearn.utils.multiclass import check_classification_targets
-    from sklearn.utils.validation import check_X_y, check_array, check_consistent_length
+    from sklearn.utils.validation import (assert_all_finite, check_X_y,
+                                          check_array, check_consistent_length)
     try:
         from sklearn.model_selection import StratifiedKFold, GroupKFold
         from sklearn.exceptions import NotFittedError
@@ -108,6 +135,7 @@ try:
     _LGBMCheckXY = check_X_y
     _LGBMCheckArray = check_array
     _LGBMCheckConsistentLength = check_consistent_length
+    _LGBMAssertAllFinite = assert_all_finite
     _LGBMCheckClassificationTargets = check_classification_targets
     _LGBMComputeSampleWeight = compute_sample_weight
 except ImportError:
@@ -122,10 +150,13 @@ except ImportError:
     _LGBMCheckXY = None
     _LGBMCheckArray = None
     _LGBMCheckConsistentLength = None
+    _LGBMAssertAllFinite = None
     _LGBMCheckClassificationTargets = None
     _LGBMComputeSampleWeight = None
 
 
 # DeprecationWarning is not shown by default, so let's create our own with higher level
 class LGBMDeprecationWarning(UserWarning):
+    """Custom deprecation warning."""
+
     pass
